﻿using ProjectDumpTruck.Drawnings;
using ProjectDumpTruck.Exceptions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace ProjectDumpTruck.CollectionGenericObject;

public class StorageCollection<T>
    where T : DrawningTruck
{
    /// <summary>
    /// Словарь (хранилище) с коллекциями
    /// </summary>
    readonly Dictionary<CollectionInfo, ICollectionGenericObject<T>> _storages;

    /// <summary>
    /// Возвращение списка названий коллекций
    /// </summary>
    public List<CollectionInfo> Keys => _storages.Keys.ToList();

    /// <summary>
    /// Ключевое слово, с которого должен начинаться файл
    /// </summary>
    private readonly string _collectionKey = "CollectionsStorage";
    /// <summary>
    /// Разделитель для записи ключа и значения элемента словаря
    /// </summary>
    private readonly string _separatorForKeyValue = "|";
    /// <summary>
    /// Разделитель для записей коллекции данных в файл
    /// </summary>
    private readonly string _separatorItems = ";";


    /// <summary>
    /// Конструктор
    /// </summary>
    public StorageCollection()
    {
        _storages = new Dictionary<CollectionInfo, ICollectionGenericObject<T>>();
    }

    /// <summary>
    /// Добавление коллекции в хранилище
    /// </summary>
    /// <param name="name">Название коллекции</param>
    /// <param name="collectionType">Тип коллекции</param>
    public void AddCollection(CollectionInfo info)
    {
        if (info == null || _storages.ContainsKey(info))
        {
            return;
        }

        if (info.CollectionType == CollectionType.Massive)
        {
            _storages.Add(info, new MassiveGenericObjects<T>());
        }

        if (info.CollectionType == CollectionType.List)
        {
            _storages.Add(info, new ListGenericObjects<T>());
        }
    }

    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="name">Название коллекции</param>
    public void DelCollection(CollectionInfo info)
    {
        if (info == null || !_storages.ContainsKey(info))
        {
            return;
        }

        _storages.Remove(info);
    }

    /// <summary>
    /// Доступ к коллекции
    /// </summary>
    /// <param name="name">Название коллекции</param>
    /// <returns></returns>
    public ICollectionGenericObject<T>? this[CollectionInfo info]
    {
        get
        {
            if (_storages.ContainsKey(info))
            {
                return _storages[info];
            }

            return null;
        }
    }

    public bool SaveData(string filename)
    {
        if (File.Exists(filename)) File.Delete(filename);

        if (_storages.Count == 0) throw new NullReferenceException("В хранилище отсутствуют коллекции для сохранения");

        using (StreamWriter sw = new StreamWriter(filename))
        {
            sw.Write(_collectionKey);

            foreach (KeyValuePair<CollectionInfo, ICollectionGenericObject<T>> value in _storages)
            {
                sw.Write(Environment.NewLine);
                // не сохраняем пустые коллекции
                if (value.Value.Count == 0) continue;

                sw.Write(value.Key);
                sw.Write(_separatorForKeyValue);
                sw.Write(value.Value.MaxCount);
                sw.Write(_separatorForKeyValue);

                foreach (T? item in value.Value.GetItems())
                {
                    string data = item?.GetDataForSave() ?? string.Empty;

                    if (string.IsNullOrEmpty(data)) continue;

                    sw.Write(data);
                    sw.Write(_separatorItems);
                }
            }
        }

        return true;
    }

    /// <summary>
    /// Загрузка информации по автомобилям в хранилище из файла
    /// </summary>
    /// <param name="filename">Путь и имя файла</param>
    /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
    public void LoadData(string filename)
    {
        if (!File.Exists(filename))
        {
            throw new FileNotFoundException("Файл не существует");
        }

        using (StreamReader sr = new(filename))
        {
            string line = sr.ReadLine();

            if (line == null || line.Length == 0)
            {
                throw new FileFormatException("В файле нет данных");
            }
            if (!line.Equals(_collectionKey))
            {
                throw new FileFormatException("В файле неверные данные");
            }
            _storages.Clear();

            while ((line = sr.ReadLine()) != null)
            {
                string[] record = line.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                if (record.Length != 3)
                {
                    continue;
                }

                CollectionInfo? collectionInfo = CollectionInfo.GetCollectionInfo(record[0]) ??
                    throw new Exception("Не удалось определить информацию коллекции: " + record[0]);
                ICollectionGenericObject<T>? collection = StorageCollection<T>.CreateCollection(collectionInfo.CollectionType) ??
                    throw new Exception("Не удалось создать коллекцию");
                collection.MaxCount = Convert.ToInt32(record[1]);

                string[] set = record[2].Split(_separatorItems, StringSplitOptions.RemoveEmptyEntries);
                foreach (string elem in set)
                {
                    if (elem?.CreateDrawningTruck() is T truck)
                    {
                        try
                        {
                            if (collection.Insert(truck, new DrawningTruckEqutables()) == -1)
                            {
                                throw new InvalidOperationException("Объект не удалось добавить в коллекцию: " + record[3]);
                            }
                        }
                        catch (CollectionOverflowException ex)
                        {
                            throw new OverflowException("Коллекция переполнена", ex);
                        }
                        catch (ObjectAlreadyInCollectionException ex)
                        {
                            throw new InvalidOperationException("Объект уже присутствует в коллекции", ex);
                        }
                    }
                }
                _storages.Add(collectionInfo, collection);
            }
        }
    }


    /// <summary>
    /// Создание коллекции по типу
    /// </summary>
    /// <param name="collectionType"></param>
    /// <returns></returns>
    private static ICollectionGenericObject<T>? CreateCollection(CollectionType collectionType)
    {
        return collectionType switch
        {
            CollectionType.Massive => new MassiveGenericObjects<T>(),
            CollectionType.List => new ListGenericObjects<T>(),
            _ => null,
        };
    }

}

