﻿using Microsoft.Extensions.Logging;
using ProjectDumpTruck.CollectionGenericObject;
using ProjectDumpTruck.Drawnings;
using ProjectDumpTruck.Exceptions;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Resources;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.TrackBar;

namespace ProjectDumpTruck;

/// <summary>
/// Форма работы с компанией и ее коллекцией
/// </summary>
public partial class FormTruckCollection : Form
{

    /// <summary>
    /// Хранилище коллекций
    /// </summary>
    private readonly StorageCollection<DrawningTruck> _storageCollection;

    /// <summary>
    /// Компания
    /// </summary>
    private AbstractCompany? _company = null;

    private readonly ILogger _logger;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormTruckCollection(ILogger<FormTruckCollection> logger)
    {
        InitializeComponent();
        _storageCollection = new();
        _logger = logger;
    }

    /// <summary>
    /// Выбор компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void СomboBoxSelectorCompany_SelectedIndexChanged_1(object sender, EventArgs e)
    {
        panelCompanyTools.Enabled = false;
    }

    /// <summary>
    /// Добавление грузовика
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddTruck_Click(object sender, EventArgs e)
    {
        FormTruckConfig form = new FormTruckConfig();
        form.Show();
        form.AddEvent(SetTruck);
    }

    /// <summary>
    /// Добавление грузовика в автомобиль
    /// </summary>
    /// <param name="truck"></param>
    private void SetTruck(DrawningTruck truck)
    {
        if (_company == null || truck == null)
        {
            return;
        }

        try
        {
           
            if (_company + truck != -1)
            {
                MessageBox.Show("Объект добавлен");
                _logger.LogInformation($"Добавлен объект {truck.GetDataForSave()}");
                pictureBox.Image = _company.Show();
            }
            else
            {
                MessageBox.Show("Не удалось добавить объект");
            }
        }
        catch (CollectionOverflowException ex)
        {
            MessageBox.Show(ex.Message);
            _logger.LogWarning($"Ошибка: {ex.Message}");
        }
        catch (ArgumentException ex)
        {
            MessageBox.Show(ex.Message);
            _logger.LogWarning($"Ошибка: {ex.Message}");
        }
    }

    /// <summary>
    /// Удаление объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRemoveTruck_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBoxPosition.Text) || _company == null)
        {
            return;
        }

        if (MessageBox.Show("Удалить объект", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }

        try
        {
            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            if (_company - pos != null)
            {
                MessageBox.Show("Объект удален");
                _logger.LogInformation($"Удален объект по позиции {pos}");
                pictureBox.Image = _company.Show();
            }
        }
        catch (ObjectNotFoundException ex)
        {
            MessageBox.Show(ex.Message);
            _logger.LogError($"Ошибка: {ex.Message}");
        }
        catch (PositionOutOfCollectionException ex)
        {
            MessageBox.Show(ex.Message);
            _logger.LogError($"Ошибка: {ex.Message}");
        }
    }

    /// <summary>
    /// Передача объекта в другую форму
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        DrawningTruck? truck = null;
        int counter = 100;
        while (truck == null)
        {
            truck = _company.GetRandomObject();
            counter--;
            if (counter <= 0)
            {
                break;
            }
        }

        if (truck == null)
        {
            return;
        }

        FormDumpTruck form = new()
        {
            SetTruck = truck
        };
        form.ShowDialog();
    }

    /// <summary>
    /// Перерисовка коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        pictureBox.Image = _company.Show();
    }

    private void ButtonCollectionAdd_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
        {
            MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            _logger.LogError("Ошибка: Заполнены не все данные для добавления коллекции");
            return;
        }

        CollectionType collectionType = CollectionType.None;
        if (radioButtonMassive.Checked)
        {
            collectionType = CollectionType.Massive;
        }
        else if (radioButtonList.Checked)
        {
            collectionType = CollectionType.List;
        }

        CollectionInfo collectionInfo = new CollectionInfo(textBoxCollectionName.Text, collectionType, string.Empty);

        _storageCollection.AddCollection(collectionInfo);
        _logger.LogInformation($"Добавлена коллекция: {textBoxCollectionName.Text} типа: {collectionType}");
        RefreshListBoxItems();
    }

    private void RefreshListBoxItems()
    {
        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i].Name;
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }
    }

    private void ButtonCollectionDel_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }

        if (MessageBox.Show("Вы уверены, что хотите удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }
        CollectionInfo collectionInfo = new CollectionInfo(listBoxCollection.SelectedItem.ToString(), CollectionType.None, string.Empty);
        _storageCollection.DelCollection(collectionInfo);
        _logger.LogInformation($"Удалена коллекция: {listBoxCollection.SelectedItem.ToString()}");
        RefreshListBoxItems();
    }

    private void ButtonCompanyCreate_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }

        CollectionInfo collectionInfo = new CollectionInfo(listBoxCollection.SelectedItem.ToString(), CollectionType.None, string.Empty);
        ICollectionGenericObject<DrawningTruck>? collection = _storageCollection[collectionInfo]; 
        if (collection == null)
        {
            MessageBox.Show("Коллекция не проинициализирована");
            return;
        }

        switch (comboBoxSelectorCompany.Text)
        {
            case "Хранилище":
                _company = new Autopark(pictureBox.Width, pictureBox.Height, collection);
                break;
        }
        panelCompanyTools.Enabled = true;
        RefreshListBoxItems();
    }

    /// <summary>
    /// Обработка нажатия "Сохранение"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (saveFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.SaveData(saveFileDialog.FileName);
                MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }

    /// <summary>
    /// Обработка нажатия "Загрузка"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (openFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.LoadData(openFileDialog.FileName);
                MessageBox.Show("Загрузка прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                RefreshListBoxItems();
                _logger.LogInformation("Загрузка из фала: {filename}", openFileDialog.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Не загрузилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }

    private void ButtonSortByType_Click(object sender, EventArgs e)
    {
        CompareTrucks(new DrawningTruckCompareByType());
    }

    private void ButtonSortByColor_Click(object sender, EventArgs e)
    {
        CompareTrucks(new DrawningTruckCompareByColor());
    }

    private void CompareTrucks(IComparer<DrawningTruck?> comparer)
    {
        if (_company == null) return;

        _company.Sort(comparer);
        pictureBox.Image = _company.Show();
    }
}


